<!-- Please do not edit this file. Edit the `blah` field in the `package.json` instead. If in doubt, open an issue. -->


[![transformer](http://i.imgur.com/2QzGSBl.png)](#)

# transformer

 [![Support me on Patreon][badge_patreon]][patreon] [![Buy me a book][badge_amazon]][amazon] [![PayPal][badge_paypal_donate]][paypal-donations] [![Ask me anything](https://img.shields.io/badge/ask%20me-anything-1abc9c.svg)](https://github.com/IonicaBizau/ama) [![Travis](https://img.shields.io/travis/IonicaBizau/transformer.svg)](https://travis-ci.org/IonicaBizau/transformer/) [![Version](https://img.shields.io/npm/v/transformer.svg)](https://www.npmjs.com/package/transformer) [![Downloads](https://img.shields.io/npm/dt/transformer.svg)](https://www.npmjs.com/package/transformer) [![Get help on Codementor](https://cdn.codementor.io/badges/get_help_github.svg)](https://www.codementor.io/johnnyb?utm_source=github&utm_medium=button&utm_term=johnnyb&utm_campaign=github)

<a href="https://www.buymeacoffee.com/H96WwChMy" target="_blank"><img src="https://www.buymeacoffee.com/assets/img/custom_images/yellow_img.png" alt="Buy Me A Coffee"></a>

> Transform data using synchronous and asynchronous functions.

## :cloud: Installation

```sh
# Using npm
npm install --save transformer

# Using yarn
yarn add transformer
```


## :clipboard: Example



```js
const Transformer = require("transformer");

let t = new Transformer({ world: "Earth" });


// There are three levels where the functions are added to be executed:
//
// Parallel:               | <0: [.............................................]>
// Unordered (don't wait): |                                <4a: [........]>
//                         +                                <4b: [....]>
//                         +                                <4c: [......]>
// Ordered (wait):         | <1: [...]> <2: [.]> <3:[.....]>                <5: [....]>

// This will run in parallel with all the other functions
t.add((data, cb) => {
    setTimeout(() => {
        data.parallel = 42;
        cb();
    }, 2000);
}, Transformer.PARALLEL);

// Async function, but ordered
t.add((data, cb) => {
    setTimeout(() => {
        data.oldWorld = data.world;
        data.world = "Mars";
        cb();
    }, 1000);
});

// Another async function and ordered
t.add((data, cb) => {
    setTimeout(() => {
        data.baz = 7;
        cb();
    }, 500);
});

// Async function, but not ordered
t.add((data, cb) => {
    setTimeout(() => {
        data.foo = 42;
        cb();
    }, 1000);
}, Transformer.UNORDERED);

// Another unordered function (this will end sooner)
t.add((data, cb) => {
    setTimeout(() => {
        data.bar = 42;
        cb(null, data);
    }, 900);
}, Transformer.UNORDERED);

// Sync function
t.add(data => {
    data.planet = data.world;
});

// Finally show the data
t.on("end", (err, data) => console.log(data));
// => { world: 'Mars'
//    , parallel: 42
//    , oldWorld: 'Earth'
//    , baz: 7
//    , bar: 42
//    , foo: 42
//    , planet: 'Mars' }
```



## :question: Get Help

There are few ways to get help:

 1. Please [post questions on Stack Overflow](https://stackoverflow.com/questions/ask). You can open issues with questions, as long you add a link to your Stack Overflow question.
 2. For bug reports and feature requests, open issues. :bug:

 3. For direct and quick help, you can [use Codementor](https://www.codementor.io/johnnyb). :rocket:



## :memo: Documentation


### `transformer(data, opts)`
Transformer
Transforms the data using synchronous and asynchronous functions.

#### Params

- **Object** `data`: The data object.
- **Object** `opts`: The options object:
 - `autostart` (Boolean): If `true`, the functions will be executed,
   without calling the `start()` method.

#### Return
- **Number** Return description.

### `add(fn, type)`
Adds a new function.

There are three levels where the functions are added to be executed:

Parallel:               | <0: [.............................................]>
Unordered (don't wait): |                                <4a: [........]>
                        +                                <4b: [....]>
                        +                                <4c: [......]>
Ordered (wait):         | <1: [...]> <2: [.]> <3:[.....]>                <5: [....]>

#### Params

- **Function|Transformer** `fn`: The function to add. Note you can add an existing transformer instance as well.
- **TransformerType** `type`: One of the following:
   - `Transformer.PARALLEL`: Used to append on the parallel timeline.
   - `Transformer.UNORDERED`: Grouped, but unordered.
   - `Transformer.ORDERED`: Grouped, but ordered.

#### Return
- **Transformer** The current Transformer instance.

### `start(data, fn)`
Starts the function execution.

#### Params

- **Object** `data`: The data object.
- **Function** `fn`: The callback function.



## :yum: How to contribute
Have an idea? Found a bug? See [how to contribute][contributing].


## :sparkling_heart: Support my projects

I open-source almost everything I can, and I try to reply to everyone needing help using these projects. Obviously,
this takes time. You can integrate and use these projects in your applications *for free*! You can even change the source code and redistribute (even resell it).

However, if you get some profit from this or just want to encourage me to continue creating stuff, there are few ways you can do it:


 - Starring and sharing the projects you like :rocket:
 - [![Buy me a book][badge_amazon]][amazon]—I love books! I will remember you after years if you buy me one. :grin: :book:
 - [![PayPal][badge_paypal]][paypal-donations]—You can make one-time donations via PayPal. I'll probably buy a ~~coffee~~ tea. :tea:
 - [![Support me on Patreon][badge_patreon]][patreon]—Set up a recurring monthly donation and you will get interesting news about what I'm doing (things that I don't share with everyone).
 - **Bitcoin**—You can send me bitcoins at this address (or scanning the code below): `1P9BRsmazNQcuyTxEqveUsnf5CERdq35V6`

    ![](https://i.imgur.com/z6OQI95.png)


Thanks! :heart:


## :dizzy: Where is this library used?
If you are using this library in one of your projects, add it in this list. :sparkles:


 - [`launchjs`](https://npmjs.com/package/launchjs)—Launch Application Framework
 - [`lien`](https://github.com/LienJS/Lien)—An easy to use web framework for Node.js.
 - [`slim-cover`](https://github.com/tsbehlman/slim-cover#readme) (by Trevor Behlman)—Barebones code coverage for JavaScript, in JavaScript

## :scroll: License

[MIT][license] © [Ionică Bizău][website]


[badge_patreon]: https://ionicabizau.github.io/badges/patreon.svg
[badge_amazon]: https://ionicabizau.github.io/badges/amazon.svg
[badge_paypal]: https://ionicabizau.github.io/badges/paypal.svg
[badge_paypal_donate]: https://ionicabizau.github.io/badges/paypal_donate.svg

[patreon]: https://www.patreon.com/ionicabizau
[amazon]: http://amzn.eu/hRo9sIZ
[paypal-donations]: https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=RVXDDLKKLQRJW

[license]: http://showalicense.com/?fullname=Ionic%C4%83%20Biz%C4%83u%20%3Cbizauionica%40gmail.com%3E%20(https%3A%2F%2Fionicabizau.net)&year=2016#license-mit
[website]: https://ionicabizau.net
[contributing]: /CONTRIBUTING.md
[docs]: /DOCUMENTATION.md
